#!/usr/bin/perl
#
# rem_vmake.pl [-fast | -slow | -quick] [ -free ]
#              [ -uselocaldev ] [ -bit64 ]
#              [ -os name ] [ -host name ]
#              [ -e cmd ]
#
#
#    ========== licence begin  GPL
#    Copyright (C) 2001 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#    ========== licence end
#


use Env;
use Getopt::Long;

#
#########################
# PTS 1107022
sub addIPAddress()
{
  chomp ( local($local_host) = `hostname` );
  local($lh_name, $lh_aliases, $lh_addrtype, $lh_length, @lh_addr) = gethostbyname($local_host);
  local($ipA,$ipB,$ipC,$ipD) = unpack('C4', $lh_addr[0]);
#  print "IP Address of $local_host = $ipA.$ipB.$ipC.$ipD\n";
  $remote_cmd .= "$ipA.$ipB.$ipC.$ipD ";
}
#
#########################
sub addLocaldev()
{
	if ( defined $uselocaldev ) {
		$local_own  = $OWN;
		$local_own  =~ s/\\/\//g;
		$remote_cmd .= "$local_own// ";
	} else {
		$remote_cmd .= "X ";
	}
}
#
#########################
sub addBit64()
{
	if ( defined $bit64 ) {
		$remote_cmd .= "BIT64 ";
	}
}
#
#########################
sub addIgmeVersion()
{
	if ( defined $ENV{IGMEVERSION} ) {
		$remote_cmd .= "IGME $ENV{IGMEVERSION} ";
	} else {
		if (  !defined($ENV{RELVER}) ) {
			print "Missing RELVER\n";
		}
		if ( !defined($ENV{CORRECTION_LEVEL}) ) {
			print "Missing RELVER\n";
		}
		if ( defined($ENV{RELVER})
		  && defined($ENV{CORRECTION_LEVEL}) ) {
			$IgmeVersion="$ENV{RELVER}";
			$IgmeVersion =~ s/R(\d)(\d+)/$1.$2/;
			if ( length($ENV{CORRECTION_LEVEL}) < 2 ) {
				$IgmeVersion .= ".0$ENV{CORRECTION_LEVEL}.develop";
			} else {
				$IgmeVersion .= ".$ENV{CORRECTION_LEVEL}.develop";
			}
			$remote_cmd .= "IGME $IgmeVersion ";
		}
	}
}
#
#########################
sub addSapdbInternal() {
    if ( !defined $ENV{SAPDB_INTERNAL} ) {
        $remote_cmd .= "NO_SAPDB_INTERNAL ";8;
    }
}

#########################
sub addAPOInfos()
{
	if ( defined $ENV{BRANCH_INFO} ) {
		$remote_cmd .= "branch=apo/$ENV{BRANCH_INFO} ";
	} else {
		print STDERR "Missing evironment variable BRANCH_INFO \n";
		die;
	}

	if ( defined $platform )
	{
		local @ARGV = ("-apo_src", $OWN, "-platform", $platform);
		do "vmakeEnv.pl";
		$lcversion=vmakeEnv::getformatedLCVERSION();
		$lc_count=vmakeEnv::substitute_LastOK();
		$remote_cmd .= "lcversion=$lcversion ";
		local $lc_count=vmakeEnv::substitute_LastOK();
		$remote_cmd .= "lcpool_count=$lc_count ";
	}
	else
	{
		if ( defined $ENV{LCVERSION} ) {
			$remote_cmd .= "lcversion=$ENV{LCVERSION} ";
		}
		elsif ( $ENV{$LCROOT} =~ /[\/\\](SAPDB_\d*_b\d\d)$/i )
		{
			$remote_cmd .= "lcversion=$1 ";
		}
		else {
			print STRERR "Missing evironment variable LCVERSION and wrong environment variable LCROOT\n";
			die;
		}
	}

	if ( defined $ENV{P4CLIENT} ) {
   		my $hostname = `hostname`;
		chomp $hostname;
		$remote_cmd .= "p4client=$ENV{P4CLIENT} " unless ( $hostname eq $ENV{P4CLIENT});
	} else {
		print STRERR "Missing evironment variable P4CLIENT\n";
		die;
	}

	if ( defined $ENV{APO_COM_SHORT} ) {
		$remote_cmd .= "apo_com_short=$ENV{APO_COM_SHORT} ";
	} else {
		print STRERR "Missing evironment variable APO_COM_SHORT \n";
		die;
	}
	if ( defined $ENV{APO_PATCH_LEVEL} ) {
		$remote_cmd .= "apo_patch_level=$ENV{APO_PATCH_LEVEL} ";
	} else {
		print STRERR "Missing evironment variable APO_PATCH_LEVEL \n";
		die;
	}
	if ( defined $ENV{RELSTAT} ) {
		$remote_cmd .= "relstat=$ENV{RELSTAT} ";
	} else {
		print STRERR "Missing evironment variable RELSTAT \n";
		die;
	}
	if (($ENV{RELSTAT} eq "PROD") && (defined $ENV{LATEST_CHANGE}) )
	{
		$remote_cmd .= "changelist=$ENV{LATEST_CHANGE} ";
	}



}

#
#########################
$fast = $slow = $quick = $free = $uselocaldev =$os = $host = $bit64 = undef;
$opt_fast = $opt_slow = $opt_quick = $opt_free = $opt_uselocaldev =$opt_os = $opt_host = $opt_bit64 = undef;

@tmp=@ARGV;
@ARGV=();
$cmd_opts=undef;
foreach (@tmp) {
        if ( ! defined $cmd_opts && $_ ne "-e") {
                push @ARGV, $_;
        } else {
				if ( !defined $cmd_opts ) {
					$cmd_opts = "";
				} else {
	                $cmd_opts=$cmd_opts." ".$_;
				}
        }
}

GetOptions ('fast!', 'slow!', 'quick!', 'free!', 'uselocaldev!', 'bit64!', 'os:s', 'host:s', 'platform:s');

$os          = $opt_os          if ( defined $opt_os );
$host        = $opt_host        if ( defined $opt_host );
$uselocaldev = $opt_uselocaldev if ( defined $opt_uselocaldev );
$bit64       = $opt_bit64       if ( defined $opt_bit64 );
$platform    = $opt_platform    if ( defined $opt_platform );


if ( !defined $host ) {
   do "$ENV{'TOOL'}\\bin\\remuser_mach$ENV{'TOOLEXT'}";
   $host = $REMUSER_HOST{$os};
}
if ( ! defined $host )
{
	print STRERR "missing remote host specification\n" ;
	die;
}

#
# build compilation mode ( only one out of fast, slow, quick survives... )
# if none is given command is handled directly...
#
if ( defined $opt_fast ) {
	$opt_slow = undef;
	$opt_quick = undef;
}
if ( defined $opt_slow ) {
	$opt_fast = undef;
	$opt_quick = undef;
}
if ( defined $opt_quick ) {
	$opt_fast = undef;
	$opt_slow = undef;
}
if ( defined $opt_free ) {
	$opt_fast = undef;
	$opt_quick = undef;
	$opt_slow = undef;
}

# PTS 1107022
if ( defined $ENV{"REM_PARALLEL"} )
{
  $title = "remuser@".$host;

  if ( defined $bit64 )
  {
	 $title .= " 64BIT";
  }
  else
  {
	 $title .= " 32BIT";
  }
  if ( defined $uselocaldev )
  {
	 $title .= " USING LOCAL SOURCES";
  }
  else
  {
	 $title .= " IGNORED OWN (NO LOCAL SOURCES)";
  }
  if ($ENV{APOMAKE})
  {
	 $title .= " apo/$ENV{BRANCH_INFO}";
     if ( defined $ENV{LCVERSION} )
	 {
        $title .= " LC=$ENV{LCVERSION}";
     }
	 elsif ( $ENV{$LCROOT} =~ /[\/\\](SAPDB_\d*_b\d\d)$/i )
	 {
		$title .= " LC=$1";
	 }
	 $title .= " $ENV{APO_COM_SHORT}";
  }
  else
  {
	if ( defined $ENV{IGMEVERSION} ) {
		$title .= " $ENV{IGMEVERSION}";
	} else {
		if ( defined($ENV{RELVER})
		  && defined($ENV{CORRECTION_LEVEL}) ) {
			$IgmeVersion="$ENV{RELVER}";
			$IgmeVersion =~ s/R(\d)(\d+)/$1.$2/;
			if ( length($ENV{CORRECTION_LEVEL}) < 2 ) {
				$IgmeVersion .= ".0$ENV{CORRECTION_LEVEL}.develop";
			} else {
				$IgmeVersion .= ".$ENV{CORRECTION_LEVEL}.develop";
			}
            $title .= " $IgmeVersion";
		}
	}
    if ( !defined $ENV{SAPDB_INTERNAL} ) {
        $title .= " NO_SAPDB_INTERNAL";
    }
  }

  if ( $ENV{"REM_PARALLEL"} eq "Start_Xterm" )
  {
	 $remote_cmd = "start \"$title\" /MIN cmd /c \"rsh $host -l remuser ";
  }
  else
  {
	 $remote_cmd = "start \"$title\" cmd /c \"rsh $host -l remuser ";
  }
}
else
{
  $remote_cmd = "rsh $host -l remuser ";
}


if ( $host eq $REMUSER_HOST{'ia64'} )
{
  $remote_cmd .= "( D:\\SAPdevelop\\remuser\\remcall_ia64.pl $USERNAME ";
}
else
{
  my $remperl;
  if ($ENV{REM_TOOLSHELL})
  { $remperl = $ENV{REM_TOOLSHELL}}
  elsif ($ENV{APOMAKE})
  { $remperl = "/devtool/local/bin/perl" }
  else
  { $remperl = "/usr/bin/perl" }

  $remote_cmd .= "( $remperl -S /rel/TOOL/tool/bin/remcall.pl $USERNAME ";
}

addIPAddress();
addLocaldev();
if ($ENV{APOMAKE})
{
	addAPOInfos();
}
else
{
	addBit64();
	addIgmeVersion();
	addSapdbInternal();
	$remote_cmd .= "imf" if ( defined $opt_fast );
	$remote_cmd .= "ims" if ( defined $opt_slow );
	$remote_cmd .= "imq" if ( defined $opt_quick );
}
$remote_cmd .= "$cmd_opts )";

if ( defined $ENV{REM_PARALLEL} )
{
  if ( $ENV{"REM_PARALLEL"} eq "Start_Xterm" )
  {
      $remote_cmd .= "\"";
  }
  else
  {
      $remote_cmd .= " && pause\"";
  }
}

if ( defined $DEBUGREM_VMAKE ) {
    print $remote_cmd;
} else {
	system($remote_cmd);
}

